from pyxyz import Confpool, H2KC, KC2H
import pyxyz.test

if __name__ == "__main__":
    pyxyz.test.run_tests()
    p = Confpool()
    p.include_from_file("crest_conformersA.xyz")
    p.include_from_file("crest_conformersB.xyz")
    print(f"Number of molecules = {p.size}")

    n_del = 0
    p["Energy"] = lambda m: float(m.descr.strip())
    n_del += p.upper_cutoff("Energy", 5.0 * KC2H)
    print(f"{n_del} confs were filtered in total")
    p.sort("Energy")  # Ascending by default. Use ascending=False to override
    p.generate_connectivity(0,
                            mult=1.3,
                            add_bonds=[[13, 23]],
                            ignore_elements=['HCarbon'])
    niso_a = p.generate_isomorphisms()
    rmsd_status = p.rmsd_filter(0.3,
                                energy_threshold=1.0 * KC2H,
                                energy_key='Energy',
                                print_status=False)
    p.generate_connectivity(0, mult=1.3, add_bonds=[[13, 23]])
    niso_b = p.generate_isomorphisms()
    print(f"{niso_b} vs. {niso_a} improvement when ignoring HCarbon atoms")
    n_del += rmsd_status['DelCount']
    print(f"{n_del} confs were filtered in total")
    p.descr = lambda m: "Conf #{}; Energy = {:6f} a.u.".format(
        m.idx + 1, m["Energy"])
    p.save_xyz('check.xyz')

    p.generate_connectivity(0,
                            mult=1.3,
                            add_bonds=[[13, 23]],
                            ignore_elements=['HCarbon'])
    p.generate_isomorphisms()
    rmsd_res = p.rmsd_filter(0.3)
    newp = Confpool()
    newp.include_subset(p,
                        [rmsd_res["MinRMSD_pairA"], rmsd_res["MinRMSD_pairB"]])
    newp.save_xyz("rmsd_test.xyz")
    print("The minimal accepted rmsd is {}".format(rmsd_res["MinRMSD"]))

    gjftext = """\
%nprocs=24
# opt=(tight,maxcycle=300) freq=noraman def2tzvp empiricaldispersion=gd3bj rpbe1pbe scrf=(cpcm,solvent=water)

Conformation_{idx}

0 1
{xyz}


"""
    atom_symbols = p.atom_symbols
    for i, m in enumerate(p):
        xyz = m.xyz  # xyz is a Numpy matrix
        xyzlines = []
        for j, symbol in enumerate(atom_symbols):
            xyzlines.append("%2s %14.6f %14.6f %14.6f" % (symbol, *xyz[j]))
        with open(f"conf_{i}.gjf", 'w') as f:
            f.write(gjftext.format(xyz="\n".join(xyzlines), idx=i))

    p["H2O dist"] = lambda m: m.l(26, 27)
    for m in p:
        print("Length = {}".format(m.l(26, 27)))
        # same as
        print("Length = {}".format(m["H2O dist"]))
    # Look at the printed numbers. And then
    print("Found {} good structures".format(
        p.count(lambda m: m["H2O dist"] < 1.8)))

    def quality_check(m):
        if m["H2O dist"] < 1.8:
            comment = "Nice"
        else:
            comment = "Garbage"
        return "{}; E={:3f} a.u.".format(comment, m["Energy"])

    # Print new descriptions in the console
    for i in range(p.size):
        print(quality_check(p[i]))
    p.descr = quality_check  # Set these new descriptions
    p.save_xyz('hbond_analysis.xyz')

    hbond_condition = lambda m: m.l(25, 10) < 2.0 and m.v(
        24, 25, 10) > 150.0 and abs(m.z(26, 23, 24, 25)) > 120.0
    num = p.count(hbond_condition)
    print("There are {} matches".format(num))
    p.filter(hbond_condition)

    min_ener = min(p["Energy"])  # p["Energy"] creates a list of floats
    p["E.Rel"] = lambda m: H2KC * (m["Energy"] - min_ener)
    p.descr = lambda m: "{}; E(rel.)={:3f} kcal/mol".format(
        m.descr, m["E.Rel"])
    p.save_xyz('extreme_hbond_analysis.xyz')

    import pandas as pd
    p = Confpool()
    p.include_from_file("crest_conformersA.xyz")
    p.include_from_file("crest_conformersB.xyz")
    p["Energy"] = lambda m: float(m.descr.strip())
    p.sort("Energy")
    p.generate_connectivity(0,
                            mult=1.3,
                            add_bonds=[[13, 23]],
                            ignore_elements=['HCarbon'])
    p.generate_isomorphisms()
    p.rmsd_filter(0.3)
    for i in range(p.size):
        if hbond_condition(p[i]) and p[i].l(26, 27) < 1.8:
            p[i]["MyCondition"] = 1.0
        else:
            p[i]["MyCondition"] = 0.0
    print(pd.DataFrame(p.as_table()))
